/****************************************************************************
** mainwin.cpp - the main window.  holds contactlist and buttons.
** Copyright (C) 2001, 2002  Justin Karneges
**
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License
** as published by the Free Software Foundation; either version 2
** of the License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
**
****************************************************************************/

#include"mainwin.h"

#include<qmessagebox.h>
#include<qiconset.h>
#include<qtooltip.h>
#include<qstyle.h>
#include<qapplication.h>
#include<qptrlist.h>
#include<qtimer.h>
#include<qobjectlist.h>
#include<qpainter.h>
#include"xmpp_types.h"
#include"common.h"
#include"showtextdlg.h"
#include"psicon.h"
#include"psiaccount.h"
#include"psitoolbar.h"
#include"ui_about.h"

#include"mainwin_p.h"

using namespace Jabber;

// deletes submenus in a popupmenu
void qpopupmenuclear(QPopupMenu *p)
{
	while(p->count()) {
		QMenuItem *item = p->findItem(p->idAt(0));
		QPopupMenu *popup = item->popup();
		p->removeItemAt(0);

		if(popup)
			delete popup;
	}
}

//*******************************************************
//  MainWin
//*******************************************************
MainWin::MainWin(bool _onTop, bool _asTool, PsiCon *_psi, const char *name)
:QMainWindow(0, name, 0 | (_onTop ? WStyle_StaysOnTop: 0) | (_asTool ? WStyle_Tool: 0))
{
	setIcon(is.status(STATUS_OFFLINE));

	psi = _psi;
	onTop = _onTop;
	asTool = _asTool;

	// sbState:
	//   -1 : connect
	// >= 0 : STATUS_*
	sbState = STATUS_OFFLINE;
	lastStatus = -2;

	nextAmount = 0;
	nextAnim = 0;
	tray = 0;
	trayMenu = 0;
	statusTip = "";
	infoString = "";
	nickname = "";

	QWidget *center = new QWidget (this, "Central widget");
	setCentralWidget ( center );

	vb_main = new QVBoxLayout(center, 2);
	cvlist = new ContactView(center);

	// needs finetuning...
	QActionGroup *viewGroups = new QActionGroup ( this );
	viewGroups->setMenuText (tr("View Groups"));
	viewGroups->setUsesDropDown (true);
	viewGroups->setExclusive (false);

	showOffline = new QAction (tr("Show Offline Contacts"), QIconSet(is.status(STATUS_OFFLINE)), tr("Show Offline Contacts"), 0, viewGroups, 0, true);
	showOffline->setOn (true);
	connect (showOffline, SIGNAL(toggled(bool)), cvlist, SLOT(setShowOffline(bool)));
	connect (cvlist, SIGNAL(showOffline(bool)), showOffline, SLOT(setOn(bool)));
	showAway = new QAction (tr("Show Away/XA/DnD"), QIconSet(is.status(STATUS_AWAY)), tr("Show Away/XA/DnD"), 0, viewGroups, 0, true);
	showAway->setOn (true);
	connect (showAway, SIGNAL(toggled(bool)), cvlist, SLOT(setShowAway(bool)));
	connect (cvlist, SIGNAL(showAway(bool)), showAway, SLOT(setOn(bool)));
	showAgents = new QAction (tr("Show Agents/Transports"), QIconSet(transport2icon("", STATUS_ONLINE)), tr("Show Agents/Transports"), 0, viewGroups, 0, true);
	showAgents->setOn (true);
	connect (showAgents, SIGNAL(toggled(bool)), cvlist, SLOT(setShowAgents(bool)));
	connect (cvlist, SIGNAL(showAgents(bool)), showAgents, SLOT(setOn(bool)));

	vb_main->addWidget(cvlist, 1);

	QHBoxLayout *hb1 = new QHBoxLayout(vb_main);
	lb_info = new MLabel(center);
	connect(lb_info, SIGNAL(clicked(int)), SLOT(statusClicked(int)));
	connect(lb_info, SIGNAL(doubleClicked()), SLOT(doRecvNextEvent()));
	lb_info->setMinimumWidth(48);
	lb_info->setSizePolicy( QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Fixed ) );
	lb_info->setFrameStyle(QFrame::Panel | QFrame::Sunken);
	hb1->addWidget(lb_info);
	lb_ssl = new QLabel(center);
	lb_ssl->setFrameStyle(QFrame::Panel | QFrame::Sunken);
	setUsingSSL(FALSE);
	hb1->addWidget(lb_ssl);
	lb_info->hide();
	lb_ssl->hide();

	add_act = new MAction(QIconSet(is.add), tr("&Add a contact"), 0, psi, this);
	connect(add_act, SIGNAL(activated(PsiAccount *, int)), SLOT(activatedAccOption(PsiAccount *, int)));

	man_act = new MAction(QIconSet(transport2icon("", STATUS_ONLINE)), tr("&Manage services"), 1, psi, this);
	connect(man_act, SIGNAL(activated(PsiAccount *, int)), SLOT(activatedAccOption(PsiAccount *, int)));

	lw_act = new MAction(QIconSet(is.xml), tr("&XML Console"), 2, psi, this);
	connect(lw_act, SIGNAL(activated(PsiAccount *, int)), SLOT(activatedAccOption(PsiAccount *, int)));

	actBrowse = new MAction(QIconSet(transport2icon("", STATUS_ONLINE)), tr("&Browse Services"), 3, psi, this);
	connect(actBrowse, SIGNAL(activated(PsiAccount *, int)), SLOT(activatedAccOption(PsiAccount *, int)));

	actDisco = new MAction(QIconSet(transport2icon("", STATUS_ONLINE)), "&Service Discovery", 4, psi, this);
	connect(actDisco, SIGNAL(activated(PsiAccount *, int)), SLOT(activatedAccOption(PsiAccount *, int)));

	actReadme = new QAction (tr("ReadMe"), tr("&ReadMe"), 0, this);
	connect (actReadme, SIGNAL(activated()), SLOT(actReadmeActivated()));
	actAbout = new QAction (tr("About"), *pix_icon_48, tr("&About"), 0, this);
	connect (actAbout, SIGNAL(activated()), SLOT(actAboutActivated()));
	actAboutQt = new QAction (tr("About Qt"), tr("About &Qt"), 0, this);
	connect (actAboutQt, SIGNAL(activated()), SLOT(actAboutQtActivated()));

	actNewMessage = new QAction (tr("New blank message"), QIconSet(is.send), tr("New &blank message"), 0, this);
	connect (actNewMessage, SIGNAL(activated()), SIGNAL(blankMessage()));
	actJoinGroupchat = new QAction (tr("Join Groupchat"), QIconSet(is.groupchat), tr("Join &Groupchat"), 0, this);
	connect (actJoinGroupchat, SIGNAL(activated()), SIGNAL(doGroupChat()));
	actAccountSetup = new QAction (tr("Account Setup"), QIconSet(is.account), tr("Acc&ount Setup"), 0, this);
	connect (actAccountSetup, SIGNAL(activated()), SIGNAL(doManageAccounts()));
	actOptions = new QAction (tr("Options"), QIconSet(is.options), tr("&Options"), 0, this);
	connect (actOptions, SIGNAL(activated()), SIGNAL(doOptions()));
	actChangeProfile = new QAction (tr("Change Profile"), QIconSet(is.profile), tr("&Change profile"), 0, this);
	connect (actChangeProfile, SIGNAL(activated()), SIGNAL(changeProfile()));
	actPlaySounds = new QAction (tr("Play sounds"), QIconSet(is.play_sounds), tr("Play &sounds"), 0, this, 0, true);
	actPlaySounds->setOn (useSound);
	connect (actPlaySounds, SIGNAL(toggled(bool)), SLOT(actPlaySoundsActivated()));
	actQuit = new QAction (tr("Quit"), QIconSet(is.quit), tr("&Quit"), 0, this);
	connect (actQuit, SIGNAL(activated()), SLOT(tryCloseProgram()));

	statusMenu = new QPopupMenu(this);
	optionsMenu = new QPopupMenu(this);
	buildStatusMenu();
	buildOptionsMenu();
	connect(statusMenu, SIGNAL(aboutToShow()), SLOT(buildStatusMenu()));
	connect(statusMenu, SIGNAL(activated(int)), SLOT(activatedStatusMenu(int)));
	connect(optionsMenu, SIGNAL(aboutToShow()), SLOT(buildOptionsMenu()));
	connect(optionsMenu, SIGNAL(activated(int)), SLOT(activatedOptionsMenu(int)));

	optionsButton = new PopupAction (tr("&Psi"), optionsMenu, this, "optionsButton");
	optionsButton->setSizePolicy ( QSizePolicy( QSizePolicy::Minimum, QSizePolicy::Minimum ) );
	optionsButton->setPixmap ( is.psimain.pixmap() );

	statusButton = new PopupAction (tr("&Status"), statusMenu, this, "statusButton");
	statusButton->setSizePolicy ( QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding ) );
	decorateButton(STATUS_OFFLINE);

#ifdef Q_WS_MAC
	vb_main->addSpacing(16);
#endif

	X11WM_CLASS("main");

	connect(psi, SIGNAL(accountCountChanged()), SLOT(numAccountsChanged()));
	numAccountsChanged();

	updateCaption();

	// create actionlist
	actions["view_groups"]  = viewGroups;
	actions["show_offline"] = showOffline;
	actions["show_away"]    = showAway;
	actions["show_agents"]  = showAgents;

	actions["button_options"] = optionsButton;
	actions["button_status"]  = statusButton;

	actions["menu_browse"]          = actBrowse;
	actions["menu_disco"]           = actDisco;
	actions["menu_add_contact"]     = add_act;
	actions["menu_new_message"]     = actNewMessage;
	actions["menu_manage_services"] = man_act;
	actions["menu_join_groupchat"]  = actJoinGroupchat;
	actions["menu_account_setup"]   = actAccountSetup;
	actions["menu_options"]         = actOptions;
	actions["menu_xml_console"]     = lw_act;
	actions["menu_change_profile"]  = actChangeProfile;
	actions["menu_play_sounds"]     = actPlaySounds;
	actions["menu_quit"]            = actQuit;

	actions["separator"] = new QAction(tr("<Separator>"), tr("<Separator>"), 0, this, "qt_separator_action");

	for (int i = 0; i < 64; i++)
		toolbars[i] = 0;

	buildToolbars ();
}

MainWin::~MainWin()
{
	if(tray) {
		delete tray;
		tray = 0;
		delete trayMenu;
		trayMenu = 0;
	}
	//saveToolbarsPositions();
	// need to find some workaround to case, when you're logging off. in that case
	// toobars are all disabled, and when you start psi again you need to enable
	// your toolbars
}

void MainWin::setWindowOpts(bool _onTop, bool _asTool)
{
	if(_onTop == onTop && _asTool == asTool)
		return;

	onTop = _onTop;
	asTool = _asTool;

	WFlags flags = 0;
	if(onTop)
		flags |= WStyle_StaysOnTop;
	if(asTool)
		flags |= WStyle_Tool;

	QPoint p = pos();
	reparent(parentWidget(), flags, p, FALSE);
	move(p);
	show();
}

void MainWin::setUseDock(bool use)
{
	if(use == false) {
		if(tray) {
			delete tray;
			tray = 0;
			delete trayMenu;
			trayMenu = 0;
		}
		return;
	}

	if(tray)
		return;

	trayMenu = new QPopupMenu;
	connect(trayMenu, SIGNAL(aboutToShow()), SLOT(buildTrayMenu()));

	tray = new MTray(is.status(STATUS_OFFLINE), "Psi", trayMenu);
	tray->setToolTip(PROG_NAME);
	connect(tray, SIGNAL(clicked(const QPoint &, int)), SLOT(trayClicked(const QPoint &, int)));
	connect(tray, SIGNAL(doubleClicked(const QPoint &)), SLOT(trayDoubleClicked()));
	connect(tray, SIGNAL(closed()), SIGNAL(closeProgram()));

	updateReadNext(nextAnim, nextAmount);

	tray->show();
}

void MainWin::setInfo(const QString &str)
{
	infoString = str;

	if(nextAmount == 0)
		lb_info->setText(QString("<nobr>") + infoString + "</nobr>");
}

void MainWin::setUsingSSL(bool use)
{
	usingSSL = use;
	lb_ssl->setPixmap(usingSSL ? is.ssl_yes : is.ssl_no);
	QToolTip::add(lb_ssl, usingSSL ? tr("Connection is encrypted") : tr("Connection is not encrypted"));
}

void MainWin::buildStatusMenu()
{
	//statusMenu->clear();
	qpopupmenuclear(statusMenu);

	statusMenu->insertItem(QIconSet(is.status(STATUS_ONLINE)),   status2txt(STATUS_ONLINE),      STATUS_ONLINE);
	statusMenu->insertItem(QIconSet(is.status(STATUS_AWAY)),     status2txt(STATUS_AWAY),        STATUS_AWAY);
	statusMenu->insertItem(QIconSet(is.status(STATUS_XA)),       status2txt(STATUS_XA),          STATUS_XA);
	statusMenu->insertItem(QIconSet(is.status(STATUS_DND)),      status2txt(STATUS_DND),         STATUS_DND);
	statusMenu->insertSeparator();
	statusMenu->insertItem(QIconSet(is.status(STATUS_INVISIBLE)),status2txt(STATUS_INVISIBLE),   STATUS_INVISIBLE);
	statusMenu->insertSeparator();
	statusMenu->insertItem(QIconSet(is.status(STATUS_OFFLINE)),  status2txt(STATUS_OFFLINE),     STATUS_OFFLINE);
}

void MainWin::activatedStatusMenu(int id)
{
	statusChanged(id);
}

void MainWin::buildToolbars()
{
	int i;
	for (i = 0; i < option.numToolbars; i++) {
		if ( toolbars[i] && !option.toolbars[i].dirty )
			continue;

		if ( toolbars[i] )
			delete toolbars[i];

		toolbars[i] = new PsiToolBar (option.toolbars[i].name, this, this);
		moveDockWindow ( toolbars[i], option.toolbars[i].dock, option.toolbars[i].nl, option.toolbars[i].index, option.toolbars[i].extraOffset );

		if ( option.toolbars[i].stretchable ) {
			toolbars[i]->setHorizontallyStretchable (true);
			toolbars[i]->setVerticallyStretchable (true);
		}

		toolbars[i]->setMovingEnabled ( !option.toolbars[i].locked );

		uint j;
		QStringList keys = option.toolbars[i].keys;
		for (j = 0; j < keys.size(); j++)
			if ( actions[keys[j]] )
				actions[keys[j]]->addTo ( toolbars[i] );

		if ( option.toolbars[i].on )
			toolbars[i]->show();
		else
			toolbars[i]->hide();

		option.toolbars[i].dirty = false;
	}

	for ( ; i < 64; i++) {
		if ( toolbars[i] )
			delete toolbars[i];

		toolbars[i] = 0;
	}
}

void MainWin::saveToolbarsPositions()
{
	for (int i = 0; i < option.numToolbars; i++) {
		getLocation ( toolbars[i], option.toolbars[i].dock, option.toolbars[i].index, option.toolbars[i].nl, option.toolbars[i].extraOffset );
		option.toolbars[i].on = toolbars[i]->isVisible();
	}
}

/*void MainWin::menuAboutToShow()
{
	// TODO: don't show context menu for toolbars for now
}*/

bool MainWin::showDockMenu(const QPoint &)
{
	return false;
}

void MainWin::buildOptionsMenu()
{
	optionsMenu->clear();

	// help menu
	QPopupMenu *helpMenu = new QPopupMenu(optionsMenu);
	
	actReadme->addTo (helpMenu);
	helpMenu->insertSeparator();
	actAbout->addTo (helpMenu);
#ifdef Q_WS_X11
	actAboutQt->addTo (helpMenu);
#endif

	// options menu
	if(!option.lockdown.roster)
		add_act->addTo(optionsMenu);
	actNewMessage->addTo (optionsMenu);
	if(!option.lockdown.services)
		actBrowse->addTo(optionsMenu);
	actJoinGroupchat->addTo (optionsMenu);
	if(!option.lockdown.accounts)
		actAccountSetup->addTo (optionsMenu);
	if(!option.lockdown.options)
		actOptions->addTo (optionsMenu);
	lw_act->addTo(optionsMenu);
	if(!option.lockdown.profiles)
		actChangeProfile->addTo (optionsMenu);
	actPlaySounds->addTo (optionsMenu);
	optionsMenu->insertSeparator();	
	optionsMenu->insertItem(QIconSet(is.help), tr("&Help"), helpMenu);
	actQuit->addTo (optionsMenu);
}

void MainWin::testme()
{
	optionsMenu->setItemEnabled(10, false);
}

void MainWin::activatedOptionsMenu(int id)
{
	QString dstr; dstr.sprintf("mainwin: id=%d\n", id);
	//pdb(DEBUG_MAINWIN, dstr);

	/*if(id == 4)
		doFileSharing();
	else if(id == 6)
		accountInfo();
	else if(id == 14) {
		// hints
	}*/
}

void MainWin::actReadmeActivated ()
{
#ifdef Q_WS_WIN
	ShowTextDlg *w = new ShowTextDlg(g.pathBase + "/readme.txt");
#else
	ShowTextDlg *w = new ShowTextDlg(g.pathBase + "/README");
#endif
	w->setCaption(CAP(tr("ReadMe")));
	w->show();
}

void MainWin::actAboutActivated ()
{
	AboutDlg *about = new AboutDlg (this, "AboutDlg", false, WDestructiveClose);
	about->show();
}

void MainWin::actAboutQtActivated ()
{
	QMessageBox::aboutQt(this);
}

void MainWin::actPlaySoundsActivated ()
{
	useSound = !useSound;
}

void MainWin::activatedAccOption(PsiAccount *pa, int x)
{
	if(x == 0)
		pa->openAddUserDlg();
	else if(x == 1)
		pa->doManageServices();
	else if(x == 2)
		pa->showXmlConsole();
	else if(x == 3)
		pa->doBrowse();
	else if(x == 4)
		pa->doDisco();
}

void MainWin::buildTrayMenu()
{
	if(!trayMenu)
		return;
	trayMenu->clear();

	if(nextAmount > 0) {
		trayMenu->insertItem(tr("Receive next event"), this, SLOT(doRecvNextEvent()));
		trayMenu->insertSeparator();
	}

	if(isHidden())
		trayMenu->insertItem(tr("Un&hide"), this, SLOT(trayShow()));
	else
		trayMenu->insertItem(tr("&Hide"), this, SLOT(trayHide()));
	optionsButton->addTo(trayMenu);
	statusButton->addTo(trayMenu);
	trayMenu->insertSeparator();
	actQuit->addTo(trayMenu);
}

void MainWin::decorateButton(int status)
{
	if(lastStatus == status)
		return;
	lastStatus = status;

	QIconSet icon;
	icon.setPixmap(is.status(status), QIconSet::Small);

	if(status == -1) {
		statusButton->setAnim(&is.conn);
		statusButton->setText(tr("Connecting"));

		setIcon(is.status(STATUS_OFFLINE));
	}
	else {
		statusButton->setAnim(0);
		statusButton->setIconSet(icon);
		statusButton->setText(status2txt(status));

		setIcon(is.status(status));
	}

	updateTray();
}

bool MainWin::askQuit()
{
	return TRUE;
}

void MainWin::tryCloseProgram()
{
	if(askQuit())
		closeProgram();
}

void MainWin::closeEvent(QCloseEvent *e)
{
	if(tray) {
		hide();
		e->accept();
		return;
	}

	if(!askQuit())
		return;

        closeProgram();
	e->accept();
}

void MainWin::keyPressEvent(QKeyEvent *e)
{
	if(tray && e->key() == Key_Escape) {
		close();
		e->accept();
		return;
	}

	QWidget::keyPressEvent(e);
}

void MainWin::updateCaption()
{
	QString str = "";

	if(nextAmount > 0)
		str += "* ";

	if(nickname.isEmpty())
		str += PROG_NAME;
	else
		str += nickname;

	if(str == caption())
		return;

	setCaption(str);
}

void MainWin::setToolbarIcons()
{
	showOffline->setIconSet(QIconSet(is.status(STATUS_OFFLINE)));
	showAway->setIconSet(QIconSet(is.status(STATUS_AWAY)));
	showAgents->setIconSet(QIconSet(transport2icon("", STATUS_ONLINE)));
}

void MainWin::optionsUpdate()
{
	int status = lastStatus;
	lastStatus = -2;
	decorateButton(status);
	setToolbarIcons();
	optionsButton->setPixmap(is.psimain);

	add_act->setIconSet (QIconSet(is.add));
	actBrowse->setIconSet (QIconSet(transport2icon("", STATUS_ONLINE)));

	updateTray();
}

void MainWin::trayClicked(const QPoint &, int button)
{
	if(option.dockDCstyle)
		return;

	if(button == MidButton) {
		doRecvNextEvent();
		return;
	}

	if(isHidden())
		trayShow();
	else
		trayHide();
}

void MainWin::trayDoubleClicked()
{
	if(!option.dockDCstyle)
		return;

	if(nextAmount > 0) {
		doRecvNextEvent();
		return;
	}

	if(isHidden())
		trayShow();
	else
		trayHide();
}

void MainWin::trayShow()
{
	show();
	bringToFront(this);
}

void MainWin::trayHide()
{
	hide();
}

void MainWin::updateReadNext(Anim *anim, int amount)
{
#ifdef Q_WS_MAC
	if(amount > 0)
		bounceDockTile();
	else
		stopDockTileBounce();
#endif

	nextAnim = anim;
	if(anim == 0)
		nextAmount = 0;
	else
		nextAmount = amount;

	if(nextAmount <= 0) {
		lb_info->hide();
		lb_info->setText("");
	}
	else {
		lb_info->setText(QString("<nobr><b>") + numEventsString(nextAmount) + "</b></nobr>");
		lb_info->show();
		// make sure it shows
		//qApp->processEvents();
	}

	updateTray();
	updateCaption();
}

QString MainWin::numEventsString(int x) const
{
	QString s;
	if(x <= 0)
		s = "";
	else if(x == 1)
		s = tr("1 event received");
	else
		s = tr("%1 events received").arg(x);

	return s;
}

void MainWin::updateTray()
{
	if(!tray)
		return;

	if(nextAmount > 0)
		tray->setAnim(nextAnim);
	else if(lastStatus == -1)
		tray->setAnim(&is.conn);
	else
		tray->setImage(is.status(lastStatus));
}

void MainWin::doRecvNextEvent()
{
	recvNextEvent();
}

void MainWin::statusClicked(int x)
{
	if(x == MidButton)
		recvNextEvent();
}

void MainWin::numAccountsChanged()
{
	bool enabled = psi->accountList().isEmpty() ? false : true;
	statusButton->setEnabled (enabled);
	//actNewMessage->setEnabled (enabled);
	//actJoinGroupchat->setEnabled (enabled);
}


#if defined(Q_WS_WIN)
#include<windows.h>
void MainWin::showNoFocus()
{
	clearWState( WState_ForceHide );

	if ( testWState(WState_Visible) ) {
		SetWindowPos(winId(),HWND_TOPMOST,0,0,0,0, SWP_NOACTIVATE|SWP_NOMOVE|SWP_NOSIZE);
		if(!onTop)
			SetWindowPos(winId(),HWND_NOTOPMOST,0,0,0,0, SWP_NOACTIVATE|SWP_NOMOVE|SWP_NOSIZE);
		return; // nothing to do
	}

	if ( isTopLevel() && !testWState( WState_Resized ) )  {
		// do this before sending the posted resize events. Otherwise
		// the layout would catch the resize event and may expand the
		// minimum size.
		QSize s = sizeHint();
		QSizePolicy::ExpandData exp;
#ifndef QT_NO_LAYOUT
		if ( layout() ) {
			if ( layout()->hasHeightForWidth() )
				s.setHeight( layout()->totalHeightForWidth( s.width() ) );
			exp =  layout()->expanding();
		} else
#endif
		{
			if ( sizePolicy().hasHeightForWidth() )
				s.setHeight( heightForWidth( s.width() ) );
			exp = sizePolicy().expanding();
		}
		if ( exp & QSizePolicy::Horizontally )
			s.setWidth( QMAX( s.width(), 200 ) );
		if ( exp & QSizePolicy::Vertically )
			s.setHeight( QMAX( s.height(), 150 ) );
		QRect screen = QApplication::desktop()->screenGeometry( QApplication::desktop()->screenNumber( pos() ) );
		s.setWidth( QMIN( s.width(), screen.width()*2/3 ) );
		s.setHeight( QMIN( s.height(), screen.height()*2/3 ) );
		if ( !s.isEmpty() )
			resize( s );
	}

	QApplication::sendPostedEvents( this, QEvent::Move );
	QApplication::sendPostedEvents( this, QEvent::Resize );

	setWState( WState_Visible );

	if ( testWFlags(WStyle_Tool) || isPopup() ) {
		raise();
	} else if ( testWFlags(WType_TopLevel) ) {
		while ( QApplication::activePopupWidget() )
		QApplication::activePopupWidget()->close();
	}

	if ( !testWState(WState_Polished) )
		polish();

	if ( children() ) {
		QObjectListIt it(*children());
		register QObject *object;
		QWidget *widget;
		while ( it ) {				// show all widget children
			object = it.current();		//   (except popups and other toplevels)
			++it;
			if ( object->isWidgetType() ) {
				widget = (QWidget*)object;
				if ( !widget->isHidden() && !widget->isTopLevel() )
				widget->show();
			}
		}
	}

#if defined(QT_ACCESSIBILITY_SUPPORT)
	QAccessible::updateAccessibility( this, 0, QAccessible::ObjectShow );
#endif

	SetWindowPos(winId(),HWND_TOP,0,0,0,0, SWP_NOACTIVATE|SWP_NOMOVE|SWP_NOSIZE|SWP_SHOWWINDOW);
	UpdateWindow(winId());
}

#else

void MainWin::showNoFocus()
{
	bringToFront(this);
}

#endif
